/*
 * Telapathy Inspector - A Telepathy client which exposes Telepathy interfaces.
 *                       Meant to inspect and/or test connection managers.
 * 
 * ti-dlg-add-members.c:
 * Dialog to add members to a Channel.Interface.Group
 * 
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
 * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "ti-dlg-add-members.h"
#include "ti-util.h"
#include "ti-config.h"

#include <glade/glade.h>

G_DEFINE_TYPE (TIDlgAddMembers, ti_dlg_add_members, G_TYPE_OBJECT);

/**
 * Instance private data.
 */
struct _TIDlgAddMembersPrivate {
    gboolean disposed;

    GtkWindow* parent;

    GladeXML* glade_xml;

    GtkWidget* dialog;

    GtkListStore* handles_list;
    GtkTreeView* handles_tree_view;
    GtkTreeSelection* handles_selection;

    GtkWidget* button_remove_handles;
    GtkEntry* entry_handle;

    GtkTextBuffer* textbuffer_message;
};
typedef struct _TIDlgAddMembersPrivate TIDlgAddMembersPrivate;

#define TI_DLG_ADD_MEMBERS_GET_PRIVATE(object)  (G_TYPE_INSTANCE_GET_PRIVATE ((object), TI_TYPE_DLG_ADD_MEMBERS, TIDlgAddMembersPrivate))

/* Function prototypes */
static void _button_ok_clicked (GtkButton *button, gpointer user_data);
static void _button_cancel_clicked (GtkButton *button, gpointer user_data);
static void _ti_dlg_add_members_build_handles_tree_view (TIDlgAddMembers* dlg_add_members);
static void _ti_dlg_add_members_update_button_remove_sensitivity (TIDlgAddMembers* dlg_add_members);
static void _ti_dlg_add_members_fill_handles_list (TIDlgAddMembers* dlg_add_members, GArray* in_handles);
static gchar* _ti_dlg_add_members_get_message_text (TIDlgAddMembers* dlg_add_members);
static void _ti_dlg_add_members_add_handle (TIDlgAddMembers* dlg_add_members);
static void _ti_dlg_add_members_setup (TIDlgAddMembers* dlg_add_members);
static void _ti_dlg_add_members_remove_handles (TIDlgAddMembers* dlg_add_members);

/**
 * Drop all references to other objects.
 */
static void
ti_dlg_add_members_dispose (GObject *object)
{
    TIDlgAddMembers *dlg_add_members = TI_DLG_ADD_MEMBERS (object);
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);

    if (priv->disposed) {
        return;
    } else {
        priv->disposed = TRUE;
    }

    if (priv->glade_xml != NULL) {
        g_object_unref (priv->glade_xml);
        priv->glade_xml = NULL;
    }

    G_OBJECT_CLASS (ti_dlg_add_members_parent_class)->dispose (object);
}

/**
 * Class initialization.
 */
static void
ti_dlg_add_members_class_init (TIDlgAddMembersClass *ti_dlg_add_members_class)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_dlg_add_members_class);

	/* override base object methods */ 
	gobject_class->dispose = ti_dlg_add_members_dispose;
		
	/* Add private */
	g_type_class_add_private (ti_dlg_add_members_class, sizeof (TIDlgAddMembersPrivate));
}

/**
 * Instance initialization.
 */
static void
ti_dlg_add_members_init (TIDlgAddMembers *ti_dlg_add_members)
{
    TIDlgAddMembersPrivate* priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (ti_dlg_add_members);

    priv->disposed = FALSE;
}

/**
 * Returns a new instance.
 */
TIDlgAddMembers*
ti_dlg_add_members_new (GtkWindow* parent) 
{
    TIDlgAddMembers* dialog = NULL;
    TIDlgAddMembersPrivate* priv = NULL;
    gchar* glade_file_path = NULL;

	dialog = g_object_new (TI_TYPE_DLG_ADD_MEMBERS, NULL);

    priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dialog);
    priv->parent = parent;

    glade_file_path = g_strdup_printf ("%s%s", TI_DATA_DIR_PREFIX, "dlg-add-members.xml");
    priv->glade_xml = glade_xml_new (glade_file_path, NULL, NULL);
    if (priv->glade_xml == NULL)
    {
        g_critical ("Error loading glade file \"%s\".", glade_file_path);
        g_object_unref (dialog);
        dialog = NULL;
        goto CLEAN_UP;
    }

    _ti_dlg_add_members_setup (dialog);

    gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), GTK_WINDOW (priv->parent));
    gtk_window_set_position (GTK_WINDOW (priv->dialog), GTK_WIN_POS_CENTER_ON_PARENT);

    glade_xml_signal_connect_data (priv->glade_xml, "button_ok_clicked", G_CALLBACK(_button_ok_clicked), priv->dialog);
    glade_xml_signal_connect_data (priv->glade_xml, "button_cancel_clicked", G_CALLBACK(_button_cancel_clicked), priv->dialog);

    CLEAN_UP:
    g_free (glade_file_path);

    return dialog;
}

/**
 * Run
 */
gboolean ti_dlg_add_members_run (TIDlgAddMembers* dlg_add_members, GArray* in_handles, GArray** out_handles, gchar** message)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    gint result;

    g_assert (out_handles != NULL);
    g_assert (message != NULL);

    gtk_list_store_clear (priv->handles_list);

    if (in_handles != NULL)
    {
        _ti_dlg_add_members_fill_handles_list (dlg_add_members, in_handles);
    }

    gtk_widget_show_all (priv->dialog);
    result = gtk_dialog_run (GTK_DIALOG (priv->dialog));
    gtk_widget_hide_all (priv->dialog);

    *message = _ti_dlg_add_members_get_message_text (dlg_add_members);

    *out_handles = ti_get_tree_model_elements (GTK_TREE_MODEL (priv->handles_list), 0, G_TYPE_UINT);

    if (*out_handles == NULL)
    {
        return FALSE;
    }
    else
    {
        return result == GTK_RESPONSE_OK;
    }
}

/**
 * Called when OK button is clicked.
 */
static void
_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_OK);
}

/**
 * Called when "Cancel" button is clicked.
 */
static void
_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
}

/**
 * Build Handles Tree View
 */
static void
_ti_dlg_add_members_build_handles_tree_view (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    priv->handles_tree_view = GTK_TREE_VIEW (glade_xml_get_widget (priv->glade_xml, "treeview_handles"));
    g_assert (priv->handles_tree_view != NULL && GTK_IS_TREE_VIEW (priv->handles_tree_view));

    priv->handles_list = gtk_list_store_new (1, G_TYPE_UINT /*handle*/);
    gtk_tree_view_set_model (priv->handles_tree_view, GTK_TREE_MODEL (priv->handles_list));

    renderer = gtk_cell_renderer_text_new ();

    column = gtk_tree_view_column_new_with_attributes ("Handle",
                                                       renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (priv->handles_tree_view, column);

    priv->handles_selection = gtk_tree_view_get_selection (priv->handles_tree_view);

    g_signal_connect_swapped (priv->handles_selection, "changed",
                              G_CALLBACK (_ti_dlg_add_members_update_button_remove_sensitivity),
                              dlg_add_members);

    gtk_tree_selection_set_mode (priv->handles_selection, GTK_SELECTION_MULTIPLE);
}

/**
 * Update Button Remove Sensitivity
 */
static void
_ti_dlg_add_members_update_button_remove_sensitivity (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    gint rows_count;

    rows_count = gtk_tree_selection_count_selected_rows (priv->handles_selection);

    gtk_widget_set_sensitive (priv->button_remove_handles, rows_count > 0);
}

/**
 * Fill Handles List
 */
static void
_ti_dlg_add_members_fill_handles_list (TIDlgAddMembers* dlg_add_members, GArray* in_handles)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    GtkTreeIter iter;
    guint i;
    guint handle;

    // Fill the list.
    for (i = 0; i < in_handles->len; i++)
    {
        handle = g_array_index (in_handles, guint, i);

        gtk_list_store_append (priv->handles_list, &iter);
        gtk_list_store_set (priv->handles_list, &iter,
                            0, handle,
                            -1);
    }
}

/**
 * Get Message Text
 */
static gchar*
_ti_dlg_add_members_get_message_text (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    GtkTextIter start_iter;
    GtkTextIter end_iter;

    gtk_text_buffer_get_start_iter (priv->textbuffer_message, &start_iter);
    gtk_text_buffer_get_end_iter (priv->textbuffer_message, &end_iter);

    return gtk_text_buffer_get_text (priv->textbuffer_message, &start_iter, &end_iter, FALSE);
}

/**
 * Add Handle
 */
static void _ti_dlg_add_members_add_handle (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    const gchar* handle_str;
    guint handle;
    int result;
    GtkTreeIter iter;
    
    handle_str = gtk_entry_get_text (priv->entry_handle);

    result = sscanf (handle_str, "%u", &handle);
    if (result != 1)
        return;

    gtk_list_store_append (priv->handles_list, &iter);
    gtk_list_store_set (priv->handles_list, &iter,
                        0, handle,
                        -1);
}

/**
 * Setup
 */
static void
_ti_dlg_add_members_setup (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);
    GtkWidget* widget;

    priv->dialog = glade_xml_get_widget (priv->glade_xml, "dialog_add_members");
    g_assert (GTK_IS_DIALOG (priv->dialog));

    // Text View Message
    widget = glade_xml_get_widget (priv->glade_xml, "textview_message");
    g_assert (widget != NULL && GTK_IS_TEXT_VIEW (widget));
    priv->textbuffer_message = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));

    // Button Remove Handles
    priv->button_remove_handles = glade_xml_get_widget (priv->glade_xml, "button_remove_handles");
    g_assert (priv->button_remove_handles != NULL && GTK_IS_BUTTON (priv->button_remove_handles));
    g_signal_connect_swapped (priv->button_remove_handles, "clicked", G_CALLBACK (_ti_dlg_add_members_remove_handles), dlg_add_members);

    // Button Add Handle
    widget = glade_xml_get_widget (priv->glade_xml, "button_add_handle");
    g_assert (widget != NULL && GTK_IS_BUTTON (widget));
    g_signal_connect_swapped (widget, "clicked", G_CALLBACK (_ti_dlg_add_members_add_handle), dlg_add_members);

    // Entry Handle
    priv->entry_handle = GTK_ENTRY (glade_xml_get_widget (priv->glade_xml, "entry_handle"));
    g_assert (priv->entry_handle != NULL && GTK_IS_ENTRY (priv->entry_handle));

    // Handles Tree View
    _ti_dlg_add_members_build_handles_tree_view (dlg_add_members);
}

/**
 * Remove Handles
 */
static void
_ti_dlg_add_members_remove_handles (TIDlgAddMembers* dlg_add_members)
{
    TIDlgAddMembersPrivate *priv = TI_DLG_ADD_MEMBERS_GET_PRIVATE (dlg_add_members);

    ti_remove_selected_elements (priv->handles_selection);
}
