/*
 * Telapathy Inspector - A Telepathy client which exposes Telepathy interfaces.
 *                       Meant to inspect and/or test handle-mapper managers.
 * 
 * ti-handle-mapper.h:
 * Maps handle numbers to their respective data (like their names)
 * 
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
 * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "ti-handle-mapper.h"
#include "ti-constants.h"

#include <dbus/dbus-glib.h>

struct _TIHandleMapperClass {
    GObjectClass parent;
};

struct _TIHandleData {
    gchar* name;
};
typedef struct _TIHandleData TIHandleData;

G_DEFINE_TYPE (TIHandleMapper, ti_handle_mapper, G_TYPE_OBJECT);

/**
 * Instance private data.
 */
struct _TIHandleMapperPrivate {
    GHashTable* hash_contact_handle_data; // Maps a contact handle with its data.
    GHashTable* hash_list_handle_data; // Maps a list handle with its data.
};
typedef struct _TIHandleMapperPrivate TIHandleMapperPrivate;

#define TI_HANDLE_MAPPER_GET_PRIVATE(object)  (G_TYPE_INSTANCE_GET_PRIVATE ((object), TI_TYPE_HANDLE_MAPPER, TIHandleMapperPrivate))

/* Function prototypes */
static void _ti_set_handle_data_name (GHashTable* hash, guint handle_number, const gchar* name);
static void _ti_handle_data_destroy (TIHandleData* handle_data);

/**
 * Drop all references to other objects.
 */
/*static void
ti_handle_mapper_dispose (GObject *object)
{
    TIHandleMapper* handle_mapper = TI_HANDLE_MAPPER (object);
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    G_OBJECT_CLASS (ti_handle_mapper_parent_class)->dispose (object);
}*/

/**
 * Finalizes the object, marking the memory as ready for reuse
 */
static void
ti_handle_mapper_finalize (GObject *object)
{
    TIHandleMapper* handle_mapper = TI_HANDLE_MAPPER (object);
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    if (priv->hash_contact_handle_data != NULL)
    {
        g_hash_table_destroy (priv->hash_contact_handle_data);
        priv->hash_contact_handle_data = NULL;
    }

    if (priv->hash_list_handle_data != NULL)
    {
        g_hash_table_destroy (priv->hash_list_handle_data);
        priv->hash_list_handle_data = NULL;
    }

	G_OBJECT_CLASS (ti_handle_mapper_parent_class)->finalize (object);
}

/**
 * Class initialization.
 */
static void
ti_handle_mapper_class_init (TIHandleMapperClass* ti_handle_mapper_class)
{
	GObjectClass* gobject_class = G_OBJECT_CLASS (ti_handle_mapper_class);

	// Override base object methods
	//gobject_class->dispose = ti_handle_mapper_dispose;
    gobject_class->finalize = ti_handle_mapper_finalize;

	// Add private
	g_type_class_add_private (ti_handle_mapper_class, sizeof (TIHandleMapperPrivate));
}

/**
 * Instance initialization.
 */
static void
ti_handle_mapper_init (TIHandleMapper* handle_mapper)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    priv->hash_contact_handle_data = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, (GDestroyNotify) _ti_handle_data_destroy);
    priv->hash_list_handle_data = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, (GDestroyNotify) _ti_handle_data_destroy);
}

/**
 * Creates a new handle mapper
 */
TIHandleMapper*
ti_handle_mapper_new ()
{
    TIHandleMapper* handle_mapper = NULL;

    handle_mapper = g_object_new (TI_TYPE_HANDLE_MAPPER, NULL);

    return handle_mapper;
}

/**
 * Set Handle Name
 */
void
ti_handle_mapper_set_handle_name (TIHandleMapper* handle_mapper, guint handle_type, guint handle_number, const gchar* name)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    if (handle_type == TI_CONNECTION_HANDLE_TYPE_CONTACT)
    {
        _ti_set_handle_data_name (priv->hash_contact_handle_data, handle_number, name);
    }
    else if (handle_type == TI_CONNECTION_HANDLE_TYPE_LIST)
    {
        _ti_set_handle_data_name (priv->hash_list_handle_data, handle_number, name);
    }
    else
    {
        g_warning ("ti_handle_mapper_set_handle_name - Unknown handle type: %u", handle_type);
    }
}

/**
 * Set Contact Handle Name
 */
void
ti_handle_mapper_set_contact_handle_name (TIHandleMapper* handle_mapper, guint handle_number, const gchar* name)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    _ti_set_handle_data_name (priv->hash_contact_handle_data, handle_number, name);
}

/**
 * Set List Handle Name
 */
void
ti_handle_mapper_set_list_handle_name (TIHandleMapper* handle_mapper, guint handle_number, const gchar* name)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    _ti_set_handle_data_name (priv->hash_list_handle_data, handle_number, name);
}

/**
 * Get Handle Name
 */
const gchar*
ti_handle_mapper_get_handle_name (TIHandleMapper* handle_mapper, guint handle_type, guint handle_number)
{
    if (handle_type == TI_CONNECTION_HANDLE_TYPE_CONTACT)
    {
        return ti_handle_mapper_get_contact_handle_name (handle_mapper, handle_number);
    }
    else if (handle_type == TI_CONNECTION_HANDLE_TYPE_LIST)
    {
        return ti_handle_mapper_get_list_handle_name (handle_mapper, handle_number);
    }
    else
    {
        g_warning ("ti_handle_mapper_get_handle_name - Unknown handle type: %u", handle_type);
        return NULL;
    }
}

/**
 * Get Contact Handle Name
 */
const gchar*
ti_handle_mapper_get_contact_handle_name (TIHandleMapper* handle_mapper, guint handle_number)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);
    TIHandleData* handle_data;

    handle_data = g_hash_table_lookup (priv->hash_contact_handle_data, &handle_number);
    if (handle_data != NULL)
    {
        return handle_data->name;
    }
    else
    {
        return NULL;
    }
}

/**
 * Get List Handle Name
 */
const gchar*
ti_handle_mapper_get_list_handle_name (TIHandleMapper* handle_mapper, guint handle_number)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);
    TIHandleData* handle_data;

    handle_data = g_hash_table_lookup (priv->hash_list_handle_data, &handle_number);
    if (handle_data != NULL)
    {
        return handle_data->name;
    }
    else
    {
        return NULL;
    }
}

/**
 * Invalidate Handle
 */
void
ti_handle_mapper_invalidate_handle (TIHandleMapper* handle_mapper, guint handle_type, guint handle_number)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);

    if (handle_type == TI_CONNECTION_HANDLE_TYPE_CONTACT)
    {
        g_hash_table_remove (priv->hash_contact_handle_data, &handle_number);
    }
    else if (handle_type == TI_CONNECTION_HANDLE_TYPE_LIST)
    {
        g_hash_table_remove (priv->hash_list_handle_data, &handle_number);
    }
    else
    {
        g_warning ("ti_handle_mapper_invalidate_handle - Unknown handle type: %u", handle_type);
    }
}

/**
 * Set Handle Data Name
 */
static void
_ti_set_handle_data_name (GHashTable* hash, guint handle_number, const gchar* name)
{
    guint *key;
    TIHandleData* handle_data;

    handle_data = g_hash_table_lookup (hash, &handle_number);

    if (handle_data == NULL)
    {
        key = g_new (guint, 1);
        *key = handle_number;

        handle_data = g_new (TIHandleData, 1);

        g_hash_table_insert (hash, key, handle_data);
    }
    else
    {
        g_free (handle_data->name);
    }

    handle_data->name = g_strdup (name);
}

/**
 * Handle Data Destroy
 */
static void _ti_handle_data_destroy (TIHandleData* handle_data)
{
    g_free (handle_data->name);
    g_free (handle_data);
}

/**
 * Invalidate Contact Handles
 */
void
ti_handle_mapper_invalidate_contact_handles (TIHandleMapper* handle_mapper, GArray* handle_numbers)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);
    guint i;
    guint handle_number;

    for (i = 0; i < handle_numbers->len; i++)
    {
        handle_number = g_array_index (handle_numbers, guint, i);
        g_hash_table_remove (priv->hash_contact_handle_data, &handle_number);
    }
}

/**
 * Invalidate List Handles
 */
void
ti_handle_mapper_invalidate_list_handles (TIHandleMapper* handle_mapper, GArray* handle_numbers)
{
    TIHandleMapperPrivate* priv = TI_HANDLE_MAPPER_GET_PRIVATE (handle_mapper);
    guint i;
    guint handle_number;

    for (i = 0; i < handle_numbers->len; i++)
    {
        handle_number = g_array_index (handle_numbers, guint, i);
        g_hash_table_remove (priv->hash_list_handle_data, &handle_number);
    }
}
